'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var tslib = require('tslib');
var nodePath = require('path');
var nodePath__default = _interopDefault(nodePath);
var write = _interopDefault(require('write'));
var pupa = _interopDefault(require('pupa'));
var chalk = _interopDefault(require('chalk'));
var spawn = _interopDefault(require('cross-spawn'));
var command = _interopDefault(require('rollup-plugin-command'));
var fs = require('fs');
var fs__default = _interopDefault(fs);
var findUp = _interopDefault(require('find-up'));
var resolve = _interopDefault(require('@rollup/plugin-node-resolve'));
var commonjs = _interopDefault(require('@rollup/plugin-commonjs'));
var makeDir = _interopDefault(require('make-dir'));

var appName = "driza";
var levels = ["info", "notice", "warn", "error", "fatal"];
var timestamp = Date.now();
var currentLogger = console.log;
var currentLevel = 1;
var debug = (function (job) {
    var log = function (messageLevel) {
        var messages = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            messages[_i - 1] = arguments[_i];
        }
        var messageLevelIndex = levels.findIndex(function (l) { return l === messageLevel; });
        if (messageLevelIndex >= currentLevel) {
            currentLogger.apply(void 0, tslib.__spreadArrays([chalk.blue.bold(appName + ":" + job)], messages, [chalk.blue.bold("+" + (Date.now() - timestamp) + "ms")]));
        }
    };
    function fatal() {
        var message = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            message[_i] = arguments[_i];
        }
        log.apply(void 0, tslib.__spreadArrays(["fatal", chalk.black.bgRed(' FATAL ERROR ')], message));
        notice('Fatal error.  Exiting with code 1.');
        process.exit(1);
    }
    function error() {
        var message = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            message[_i] = arguments[_i];
        }
        log.apply(void 0, tslib.__spreadArrays(["error", chalk.black.bgRed(' ERROR ')], message));
    }
    function warn() {
        var message = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            message[_i] = arguments[_i];
        }
        log.apply(void 0, tslib.__spreadArrays(["warn", chalk.black.bgYellow(' WARN ')], message));
    }
    function notice() {
        var message = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            message[_i] = arguments[_i];
        }
        log.apply(void 0, tslib.__spreadArrays(["notice", chalk.black.bgCyan(' NOTICE ')], message));
    }
    function info() {
        var message = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            message[_i] = arguments[_i];
        }
        log.apply(void 0, tslib.__spreadArrays(["info", chalk.black.bgGreen(' INFO ')], message));
    }
    return {
        fatal: fatal,
        error: error,
        warn: warn,
        notice: notice,
        info: info,
    };
});

var template = "<!DOCTYPE html>\n<html lang=\"en\">\n\t<head>\n\t\t<!-- Set the default title -->\n\t\t<title>Loading...</title>\n\n\t\t<!-- Meta -->\n\t\t<meta charset=\"utf-8\" />\n\t\t<meta name=\"viewport\" content=\"width=device-width,initial-scale=1\" />\n\t\t<meta http-equiv=\"Content-Security-Policy\" content=\"script-src 'self';\" />\n\n\t\t<!-- Build script generated by rollup -->\n\t\t<script defer src=\"{appEntry}\"></script>\n\t</head>\n\n\t<body></body>\n</html>\n";

var log = debug('actions');
function writeTemplate(dir, options) {
    return tslib.__awaiter(this, void 0, void 0, function () {
        return tslib.__generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, write(nodePath__default.join(dir, 'index.html'), pupa(template, options))];
                case 1:
                    _a.sent();
                    log.info("Migrated template to index.html");
                    return [2 /*return*/];
            }
        });
    });
}
function writeJSONBuildOptions(dir, options) {
    return tslib.__awaiter(this, void 0, void 0, function () {
        return tslib.__generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, write(nodePath__default.join(dir, 'build-options.json'), JSON.stringify(options))];
                case 1:
                    _a.sent();
                    log.info("Migrated options to build-options.json");
                    return [2 /*return*/];
            }
        });
    });
}

var plugin = (function (buildOptions, options) {
    var dir = nodePath__default.join(buildOptions.outDir, options.tag);
    return {
        name: 'electron',
        buildStart: function () { return tslib.__awaiter(void 0, void 0, void 0, function () {
            var pkg, writePkg, actions;
            return tslib.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        pkg = {
                            name: buildOptions.id,
                            main: options.mainBundlePath,
                        };
                        writePkg = write(nodePath__default.join(dir, 'package.json'), JSON.stringify(pkg));
                        actions = [
                            writeTemplate(dir, { name: buildOptions.name, appEntry: options.bundlePath }),
                            writeJSONBuildOptions(dir, buildOptions),
                            writePkg,
                        ];
                        return [4 /*yield*/, Promise.all(actions)];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        }); },
    };
});

var logChild = function (errorLog, dataLog, child, fatalLog) {
    if (fatalLog === void 0) { fatalLog = errorLog; }
    child.stdout.on('data', function (data) { return dataLog(data.toString().trim()); });
    child.stderr.on('data', function (data) { return errorLog(data.toString().trim()); });
    child.on('close', function (code) {
        if (code)
            fatalLog("A child process exited with a non-0 exit code.");
    });
    process.on('exit', function () {
        child.kill('SIGINT');
    });
};

var log$1 = debug('run:electron');
var runElectron = (function (options, buildOptions, mainBundlePath) { return function () {
    return command(function () {
        log$1.notice('Launching application...');
        var child = spawn(options.pathToElectron, [nodePath__default.join(buildOptions.outDir, options.tag, mainBundlePath)]);
        logChild(log$1.error, log$1.notice, child, log$1.fatal);
    }, { once: true });
}; });

var fileExists = (function (file) {
    return new Promise(function (resolve) {
        if (!file)
            resolve();
        fs.access(file, fs.constants.F_OK, function (err) {
            if (err)
                resolve();
            else
                resolve(file);
        });
    });
});

var nativeNodeModules = [
    "process",
    "events",
    "stream",
    "util",
    "path",
    "buffer",
    "querystring",
    "url",
    "string_decoder",
    "punycode",
    "http",
    "https",
    "os",
    "assert",
    "constants",
    "timers",
    "console",
    "vm",
    "zlib",
    "tty",
    "domain",
    "dns",
    "dgram",
    "child_process",
    "cluster",
    "module",
    "net",
    "readline",
    "repl",
    "tls",
    "fs",
    "crypto",
];

var defaultElectronMain = "import { app } from 'electron';\nimport { createBrowserWindow } from 'driza';\nimport { is } from 'electron-util';\n\ntry {\n\trequire('electron-reloader')(module);\n} catch (_) {}\n\nlet mainWindow;\n\nfunction createWindow() {\n\tmainWindow = createBrowserWindow({\n\t\twebPreferences: {\n\t\t\tnodeIntegration: true,\n\t\t},\n\t\tshow: false,\n\t});\n\n\tif (is.development) mainWindow.webContents.openDevTools();\n\n\tmainWindow.on('ready-to-show', mainWindow.show);\n\n\tmainWindow.on('closed', () => {\n\t\tmainWindow = null;\n\t});\n}\n\napp.on('ready', createWindow);\n\napp.on('window-all-closed', () => {\n\tif (process.platform !== 'darwin') app.quit();\n});\n\napp.on('activate', () => {\n\tif (mainWindow === null) createWindow();\n});\n";

var platform = function (options) {
    if (options === void 0) { options = {}; }
    return function (buildOptions, debug) { return tslib.__awaiter(void 0, void 0, void 0, function () {
        var log, defaultMainPath, errorPrefix, electronDoesExist, pathToElectron;
        return tslib.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    log = debug('electron');
                    // Set the default options
                    options = {
                        pathToElectron: options.pathToElectron,
                        bundlePath: options.bundlePath || "app.js",
                        mainBundlePath: options.mainBundlePath || "index.js",
                        rollupConfig: options.rollupConfig,
                        // entryFile will be set later
                        tag: options.tag || "electron",
                    };
                    if (!!options.entryFile) return [3 /*break*/, 2];
                    defaultMainPath = nodePath__default.join(buildOptions.outDir, options.tag, "__pre-index.js");
                    return [4 /*yield*/, write(defaultMainPath, defaultElectronMain)];
                case 1:
                    _a.sent();
                    log.info("options.entryFile not specified, writing a default '__pre-main.js' file instead.");
                    options.entryFile = defaultMainPath;
                    _a.label = 2;
                case 2:
                    // Set the rollup configuration for bundling the electron main process
                    options.rollupConfig = options.rollupConfig || {
                        input: options.entryFile,
                        output: {
                            file: nodePath__default.join(buildOptions.outDir, options.tag, options.mainBundlePath),
                            format: 'commonjs',
                        },
                        plugins: [resolve({ preferBuiltins: true }), commonjs()],
                        external: nativeNodeModules.concat(['electron']),
                        onwarn: function (warning) {
                            debug('bundle').warn("(Rollup bundle electron main process)", warning.message);
                        },
                    };
                    errorPrefix = "Electron could not be found.";
                    if (!options.pathToElectron) return [3 /*break*/, 4];
                    return [4 /*yield*/, fileExists(options.pathToElectron)];
                case 3:
                    electronDoesExist = _a.sent();
                    if (!electronDoesExist)
                        log.fatal(errorPrefix, "\"" + options.pathToElectron + "\" doesn't exist.");
                    _a.label = 4;
                case 4: return [4 /*yield*/, findUp("node_modules/.bin/electron")];
                case 5:
                    pathToElectron = _a.sent();
                    if (!pathToElectron)
                        log.fatal(errorPrefix);
                    options.pathToElectron = pathToElectron;
                    // Return the hooks and values
                    return [2 /*return*/, {
                            tag: options.tag,
                            data: "electron",
                            plugin: function () { return plugin(buildOptions, options); },
                            external: function () { return ['electron']; },
                            extraBuilds: function () { return [options.rollupConfig]; },
                            isSandboxed: function () { return false; },
                            bundlePath: function () { return options.bundlePath; },
                            runtimePath: function () { return "runtime.js"; },
                            run: runElectron(options, buildOptions, options.mainBundlePath),
                        }];
            }
        });
    }); };
};

var log$2 = debug('read-files');
var read = function () {
    var paths = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        paths[_i] = arguments[_i];
    }
    return new Promise(function (resolve) {
        var path = nodePath__default.join.apply(nodePath__default, paths);
        fs__default.readFile(path, 'utf-8', function (err, data) {
            if (err)
                return resolve("");
            return resolve(data);
        });
    });
};
var readJSON = function () {
    var paths = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        paths[_i] = arguments[_i];
    }
    return tslib.__awaiter(void 0, void 0, Promise, function () {
        var res;
        return tslib.__generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, read.apply(void 0, paths)];
                case 1:
                    res = _a.sent();
                    try {
                        return [2 /*return*/, JSON.parse(res.length ? res : "{}")];
                    }
                    catch (e) {
                        log$2.error('Failed to load JSON', e);
                        return [2 /*return*/, {}];
                    }
                    return [2 /*return*/];
            }
        });
    });
};

var bugs = {
	url: "https://github.com/Vehmloewff/driza/issues/new"
};

var configXml = "<?xml version='1.0' encoding='utf-8'?>\n<widget\n\tid=\"{bundleId}\"\n\tversion=\"{version}\"\n\txmlns=\"http://www.w3.org/ns/widgets\"\n\txmlns:cdv=\"http://cordova.apache.org/ns/1.0\"\n>\n\t<name>{name}</name>\n\t<description>{description}</description>\n\t<author email=\"{author.email}\" href=\"{author.url}\">{author.name}</author>\n\t<icon src=\"../../{icon.path}\" />\n\t<content src=\"index.html\" />\n\t<plugin name=\"cordova-plugin-whitelist\" spec=\"\" />\n\t<access origin=\"*\" />\n\t<allow-intent href=\"http://*/*\" />\n\t<allow-intent href=\"https://*/*\" />\n\t<allow-intent href=\"tel:*\" />\n\t<allow-intent href=\"sms:*\" />\n\t<allow-intent href=\"mailto:*\" />\n\t<allow-intent href=\"geo:*\" />\n\t<platform name=\"android\">\n\t\t<allow-intent href=\"market:*\" />\n\t</platform>\n\t<platform name=\"ios\">\n\t\t<allow-intent href=\"itms:*\" />\n\t\t<allow-intent href=\"itms-apps:*\" />\n\t</platform>\n</widget>\n";

var log$3 = debug('prepare:cordova');
var plugin$1 = (function (options, runnable, platform, appEntry, dir) {
    var didChangeThings = false;
    function addCordovaPlatform() {
        var _this = this;
        didChangeThings = true;
        return new Promise(function (resolve) {
            log$3.notice("Adding cordova-" + platform + "...");
            var child = spawn(runnable, ["platform", "add", platform], {
                cwd: dir,
            });
            logChild(log$3.error, log$3.info, child);
            child.on('close', function (code) { return tslib.__awaiter(_this, void 0, void 0, function () {
                return tslib.__generator(this, function (_a) {
                    switch (_a.label) {
                        case 0:
                            if (code)
                                log$3.fatal("Failed to add cordova-" + platform + ".");
                            log$3.notice("Done!");
                            return [4 /*yield*/, install()];
                        case 1:
                            _a.sent();
                            resolve();
                            return [2 /*return*/];
                    }
                });
            }); });
            function install() {
                return tslib.__awaiter(this, void 0, void 0, function () {
                    var child, errors;
                    return tslib.__generator(this, function (_a) {
                        log$3.info("Installing dependencies via npm...");
                        child = spawn("npm", ["install"], {
                            cwd: dir,
                        });
                        errors = [];
                        logChild(function (error) { return errors.push(error); }, function () { }, child);
                        child.on('close', function (code) {
                            if (code) {
                                log$3.error("We are having trouble installing the dependencies.");
                                log$3.error(errors.join('\n'));
                                log$3.fatal("Failed install cordova-" + platform + " from npm.");
                            }
                            log$3.info("Installed dependencies.");
                            resolve();
                        });
                        return [2 /*return*/];
                    });
                });
            }
        });
    }
    function cordovaPrepare() {
        return new Promise(function (resolve) {
            var child = spawn(runnable, ["prepare"], {
                cwd: dir,
            });
            logChild(log$3.error, log$3.notice, child);
            child.on('close', function (code) {
                if (code)
                    log$3.fatal("Cordova is having problems with what we are doing.  Would you consider opening an issue?  " + bugs.url);
                resolve();
            });
        });
    }
    return {
        name: 'cordova',
        buildStart: function () { return tslib.__awaiter(void 0, void 0, void 0, function () {
            var pkg, actions, platformIsInstalled;
            return tslib.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, readJSON(dir, 'package.json')];
                    case 1:
                        pkg = _a.sent();
                        actions = [
                            writeTemplate(nodePath.join(dir, 'www'), { appEntry: appEntry }),
                            writeJSONBuildOptions(dir, options),
                            makeDir(nodePath.join(dir, 'platforms')),
                            write(nodePath.join(dir, "config.xml"), pupa(configXml, options)),
                        ];
                        if (!!Object.keys(pkg).length) return [3 /*break*/, 3];
                        didChangeThings = true;
                        pkg = {
                            name: options.id,
                        };
                        return [4 /*yield*/, write(nodePath.join(dir, 'package.json'), JSON.stringify(pkg))];
                    case 2:
                        _a.sent();
                        _a.label = 3;
                    case 3:
                        platformIsInstalled = pkg.cordova && pkg.cordova.platforms && pkg.cordova.platforms.find(function (p) { return p === platform; });
                        if (!!platformIsInstalled) return [3 /*break*/, 5];
                        return [4 /*yield*/, addCordovaPlatform()];
                    case 4:
                        _a.sent();
                        _a.label = 5;
                    case 5:
                        if (!didChangeThings) return [3 /*break*/, 7];
                        return [4 /*yield*/, cordovaPrepare()];
                    case 6:
                        _a.sent();
                        _a.label = 7;
                    case 7: return [4 /*yield*/, Promise.all(actions)];
                    case 8:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        }); },
    };
});

var log$4 = debug('run:android');
var runAndroid = (function (options, buildOptions, platform) { return function () {
    return command(function () {
        log$4.notice('Building binaries...');
        var child = spawn(options.pathToCordova, ['run', platform, "--debug", "--stacktrace"], {
            cwd: nodePath__default.join(buildOptions.outDir, options.tag),
        });
        logChild(log$4.error, function (message) {
            if (/BUILD SUCCESSFUL/.test(message))
                log$4.notice("Launching app...");
            if (/LAUNCH SUCCESS/.test(message))
                log$4.notice("App lanuched! \uD83D\uDE80 \uD83D\uDE80 \uD83D\uDE80 \uD83D\uDE80 ");
            log$4.info(message);
        }, child);
    });
}; });

var platform$1 = function (options) {
    if (options === void 0) { options = {}; }
    return function (buildOptions, debug) { return tslib.__awaiter(void 0, void 0, void 0, function () {
        var log, errorPrefix, electronDoesExist, pathToCordova, result;
        return tslib.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    log = debug("android");
                    // Set the default options
                    options = {
                        pathToCordova: options.pathToCordova,
                        bundlePath: options.bundlePath || "app.js",
                        tag: options.tag || "android",
                    };
                    errorPrefix = "Cordova could not be found.";
                    if (!options.pathToCordova) return [3 /*break*/, 2];
                    return [4 /*yield*/, fileExists(options.pathToCordova)];
                case 1:
                    electronDoesExist = _a.sent();
                    if (!electronDoesExist)
                        log.fatal(errorPrefix, "\"" + options.pathToCordova + "\" doesn't exist.");
                    _a.label = 2;
                case 2: return [4 /*yield*/, findUp("node_modules/.bin/cordova")];
                case 3:
                    pathToCordova = _a.sent();
                    if (!pathToCordova)
                        log.fatal(errorPrefix);
                    options.pathToCordova = pathToCordova;
                    result = {
                        tag: "android",
                        isSandboxed: function () { return true; },
                        bundlePath: function () { return nodePath__default.join("www", options.bundlePath); },
                        runtimePath: function () { return "www/runtime.js"; },
                        plugin: function () {
                            return plugin$1(buildOptions, options.pathToCordova, "android", options.bundlePath, nodePath__default.join(buildOptions.outDir, options.tag));
                        },
                        run: runAndroid(options, buildOptions, "android"),
                    };
                    return [2 /*return*/, result];
            }
        });
    }); };
};

exports.android = platform$1;
exports.electron = platform;
